@@ -10,7 +10,7 @@ with 'MooX::Singleton';
 
 memoize('compile_marc_path');
 
-our $VERSION = '1.04';
+our $VERSION = '1.05';
 
 sub marc_map {
     my $self      = $_[0];
@@ -565,6 +565,8 @@ Catmandu::MARC - Catmandu modules for working with MARC data
 
 =item * L<Catmandu::Fix::marc_map>
 
+=item * L<Catmandu::Fix::marc_spec>
+
 =item * L<Catmandu::Fix::marc_add>
 
 =item * L<Catmandu::Fix::marc_remove>
@@ -609,7 +611,8 @@ processing at L<https://github.com/LibreCat/Catmandu/wiki>.
 L<Catmandu>,
 L<Catmandu::Importer>,
 L<Catmandu::Fix>,
-L<Catmandu::Store>
+L<Catmandu::Store>,
+L<MARC::Spec>
 
 =head1 AUTHOR
 
@@ -629,6 +632,8 @@ Patrick Hochstenbach, C<< <patrick.hochstenbach at ugent.be> >>
 
 =item * Robin Sheat
 
+=item * Carsten Klee, C<< klee at cpan.org >>
+
 =back
 
 =head1 LICENSE AND COPYRIGHT
